function [ids_core_sources,ids_core_sources_description,varargout] = ...
  tcv_get_ids_core_sources(shot,ids_equil_empty,gdat_params,varargin)
%
% [ids_core_sources,ids_core_sources_description,varargout] = ...
%     tcv_get_ids_core_sources(shot,ids_equil_empty,gdat_params,varargin);
%
%
% gdat_params: gdat_data.gdat_params to get all params passed from original call
%

machine = 'tcv';
tens_time = -1;
tens_rho = -0.1;

if exist('gdat_params','var')
  [ids_core_sources, params_core_sources] = ...
    tcv_ids_headpart(shot,ids_equil_empty,'cores_sources','gdat_params',gdat_params,...
    'homogeneous_time',0,varargin{:});
else
  [ids_core_sources, params_core_sources] = ...
    tcv_ids_headpart(shot,ids_equil_empty,'cores_sources','homogeneous_time',0,varargin{:});
  aa=gdat_tcv;
  gdat_params = aa.gdat_params; % to get default params
end
params_eff_ref = gdat_params; params_eff_ref.doplot=0;
try params_eff_ref=rmfield(params_eff_ref,'source');catch;end % make sure no source (from ids def)

% initialize description
ids_core_sources_description = [];

%% fill vacuum_toroidal field
params_eff = params_eff_ref; params_eff.data_request='b0';
b0_gdat=gdat(params_core_sources.shot,params_eff);
ids_core_sources.vacuum_toroidal_field.r0 = b0_gdat.r0;
ids_core_sources.vacuum_toroidal_field.b0 = b0_gdat.data;
ids_core_sources_description.vacuum_toroidal_field = b0_gdat.request_description;
ids_core_sources.time = b0_gdat.t;
%%
% save source template to fill for differnet source options
source_template = ids_core_sources.source{1};
profiles_template = source_template.profiles_1d{1};
globals_template = source_template.global_quantities{1};

last_index = 0; % index to be incremented when sources are added

%% get time grid and data from gdat
params_eff = params_eff_ref; params_eff.data_request='ohm_data';
ohm_gdat = gdat(params_core_sources.shot,params_eff);
params_eff = params_eff_ref; params_eff.data_request='powers';
powers_gdat = gdat(params_core_sources.shot,params_eff);
%% initialize source from template
% ohm
ohm_t_grid = powers_gdat.ohm.t; ohm_n_t = numel(ohm_t_grid);

main_desc = 'Source from ohmic heating'; production = 'IBS nodes';
id_ohm.description = sprintf('%s from %s',main_desc,production);
id_ohm.index = 7; id_ohm.name = 'ohmic';
ids_core_sources.source{last_index+1} = source_template;
ids_core_sources.source{last_index+1}.identifier = id_ohm;
ids_core_sources.source{last_index+1}.profiles_1d(1:ohm_n_t) = {profiles_template};
ids_core_sources.source{last_index+1}.global_quantities = globals_template;
ohm_data = ohm_gdat.ohm.ohm_data;

% load LIUQE data to convert
% \tilde{j}_// = <jdotB>/(R0<Bphi/R>) to j_//0 = <jdotB>/B0; Bphi=T(psi)/R
[L,~,LY] = liuqe(shot,ohm_t_grid,'iterq',50,'ilim',3,'icsint',true);
rho_pol_norm_liu = L.pQ; % normalized sqrt poloidal flux including axis
T      = LY.TQ;
Rm2_fs = LY.Q2Q;
vol    = LY.VQ;
R0 = ids_core_sources.vacuum_toroidal_field.r0;
B0 = interp1(ids_core_sources.time,ids_core_sources.vacuum_toroidal_field.b0,ohm_t_grid);
jpar_tilde_to_jpar_0 = R0*T.*Rm2_fs./B0';

% map volume and conversion factor on rho_pol grid of ohm_data cd_dens
jpar_tilde_to_jpar_0_mapped = interp1(rho_pol_norm_liu,jpar_tilde_to_jpar_0,ohm_data.cd_dens.rhopol_norm');
vol_mapped = interp1(rho_pol_norm_liu,vol,ohm_data.cd_dens.rhopol_norm');

% profiles_1d
current_parallel_tmp = nan(1,ohm_n_t);
for ii = 1:ohm_n_t
  ids_core_sources.source{last_index+1}.profiles_1d{ii}.time = ohm_t_grid(ii);
  ids_core_sources.source{last_index+1}.profiles_1d{ii}.grid.rho_pol_norm = ...
    ohm_data.cd_dens.rhopol_norm';
  ids_core_sources.source{last_index+1}.profiles_1d{ii}.grid.rho_tor_norm = ...
    ohm_data.cd_dens.rhotor_norm(:,ii)';
  jpar_0_tmp = ohm_data.cd_dens.data(:,ii)'.*jpar_tilde_to_jpar_0_mapped(:,ii)';
  ids_core_sources.source{last_index+1}.profiles_1d{ii}.j_parallel = jpar_0_tmp;
  integrated_jpar_0_tmp = cumtrapz(vol_mapped(:,ii),jpar_0_tmp)/2/pi/R0;
  ids_core_sources.source{last_index+1}.profiles_1d{ii}.current_parallel_inside = integrated_jpar_0_tmp;
  current_parallel_tmp(ii) = integrated_jpar_0_tmp(end);
end

% globals
ids_core_sources.source{last_index+1}.global_quantities.time = ohm_t_grid;
ids_core_sources.source{last_index+1}.global_quantities.power = powers_gdat.ohm.data;
ids_core_sources.source{last_index+1}.global_quantities.current_parallel = current_parallel_tmp;

last_index = last_index+1;  % add if statement to only increment if ohmic source has been added

%% bs
params_eff = params_eff_ref; params_eff.data_request='bs_data';
bs_gdat = gdat(params_core_sources.shot,params_eff); bs_data = bs_gdat.bs.bs_data;
bs_t_grid = bs_gdat.bs.t; bs_n_t = numel(bs_t_grid);

main_desc = 'Bootstrap current'; production = 'IBS nodes';
id_bs.description = sprintf('%s from %s',main_desc,production);
id_bs.index = 13; id_bs.name = 'bootstrap_current';
ids_core_sources.source{last_index+1} = source_template;
ids_core_sources.source{last_index+1}.identifier = id_bs;
ids_core_sources.source{last_index+1}.profiles_1d(1:bs_n_t) = {profiles_template};
ids_core_sources.source{last_index+1}.global_quantities = globals_template;

% profiles_1d
current_parallel_tmp = nan(1,bs_n_t);
for ii = 1:bs_n_t
  ids_core_sources.source{last_index+1}.profiles_1d{ii}.time = bs_t_grid(ii);
  ids_core_sources.source{last_index+1}.profiles_1d{ii}.grid.rho_pol_norm = ...
    bs_data.cd_dens.rhopol_norm';
  ids_core_sources.source{last_index+1}.profiles_1d{ii}.grid.rho_tor_norm = ...
    bs_data.cd_dens.rhotor_norm(:,ii)';
  % use same conversion as for ohmic data from LIUQE
  jpar_0_tmp = bs_data.cd_dens.data(:,ii)'.*jpar_tilde_to_jpar_0_mapped(:,ii)';
  ids_core_sources.source{last_index+1}.profiles_1d{ii}.j_parallel = jpar_0_tmp;
  integrated_jpar_0_tmp = cumtrapz(vol_mapped(:,ii),jpar_0_tmp)/2/pi/R0;
  ids_core_sources.source{last_index+1}.profiles_1d{ii}.current_parallel_inside = integrated_jpar_0_tmp;
  current_parallel_tmp(ii) = integrated_jpar_0_tmp(end);
end

% globals
ids_core_sources.source{last_index+1}.global_quantities.time = bs_t_grid;
ids_core_sources.source{last_index+1}.global_quantities.current_parallel = current_parallel_tmp;

last_index = last_index+1;  % add if statement to only increment if bs source has been added

%% ec
params_eff = params_eff_ref;
params_eff.data_request='ec_data';
params_eff.ec_inputs = 1; % load EC input information
ec_gdat = gdat(params_core_sources.shot,params_eff);

if ~isempty(ec_gdat.ec.data) % if EC data available, fill sources

  ec_data = ec_gdat.ec.ec_data; ec_inputs = ec_gdat.ec.ec_inputs;
  % get tgrid from gdat ec_data
  ec_tgrid = ec_data.p_dens.t; nt_ec_data = numel(ec_tgrid);

  % retrieve active launcher information from ec_inputs
  nb_launchers = numel(ec_inputs.launchers_active.data);
  active_launchers = find(ec_inputs.launchers_active.data==1)';

  % find times of injected EC power to interpolate power & current densities
  % on p_ec_injected tgrid
  ec_powers_tgrid = powers_gdat.ec.t; %nt_ec_powers = numel(ec_powers_tgrid);
  itime_ec = powers_gdat.ec.data(:,end)>0;
  interp_tgrid = ec_powers_tgrid(itime_ec); nt_interp = numel(interp_tgrid);
  interp_p_ec_injected = powers_gdat.ec.data(itime_ec,end);

  % Setup source structs for active launchers from template
  main_desc = 'Source from electron cyclotron heating and current drive';
  production = 'TORAY';
  id_ec.index = 3; id_ec.name = 'ec';
  for i_lau = active_launchers
    id_ec.description = sprintf('L%i/G%i, %s from %s double width CD profiles',i_lau,ec_inputs.gyro2launcher.data(i_lau),main_desc,production);
    ids_core_sources.source{last_index+i_lau} = source_template;
    ids_core_sources.source{last_index+i_lau}.identifier = id_ec;
    ids_core_sources.source{last_index+i_lau}.profiles_1d(1:nt_interp) = {profiles_template};
    ids_core_sources.source{last_index+i_lau}.global_quantities = globals_template;
  end

  % load LIUQE data to convert
  % \tilde{j}_// = <jdotB>/(R0<Bphi/R>) to j_//0 = <jdotB>/B0; Bphi=T(psi)/R
  [L,~,LY] = liuqe(shot,ec_tgrid,'iterq',50,'ilim',3,'icsint',true);
  rho_pol_norm_liu = L.pQ; % normalized sqrt poloidal flux including axis
  mu0     = 4*pi*10^-7;
  Rm2_fs  = LY.Q2Q;
  mVprime = 1./LY.Q1Q; % Q1Q: -dpsi/dV
  Ip_psi  = LY.ItQ; % Ip(psi) = I_tor(psi)
  T       = LY.TQ;
  vol     = LY.VQ;
  B2_fs   = mu0*Ip_psi./mVprime + T.^2.*Rm2_fs;
  R0 = ids_core_sources.vacuum_toroidal_field.r0;
  B0 = interp1(ids_core_sources.time,ids_core_sources.vacuum_toroidal_field.b0,ec_tgrid);
  jtoray_to_jpar0 = 2*pi./B0./(T.*Rm2_fs).*B2_fs;

  % interpolate on ec_data rho_pol grid
  jtoray_to_jpar0_mapped = nan(size(ec_data.cd_dens.x));
  vol_mapped = nan(size(ec_data.cd_dens.x));
  for ii = 1:nt_ec_data
    jtoray_to_jpar0_mapped(:,ii) = ...
      interp1(rho_pol_norm_liu,jtoray_to_jpar0(:,ii),ec_data.cd_dens.grids.rho_pol_norm(:,ii));
    vol_mapped(:,ii) = ...
      interp1(rho_pol_norm_liu,vol(:,ii),ec_data.cd_dens.grids.rho_pol_norm(:,ii));
  end

  % load 1d_profiles from ec_data
  p_dens        = ec_data.p_dens.data;
  p_integrated  = ec_data.p_integrated.data;
  cd_dens       = ec_data.cd_dens_doublewidth.data; % use double width to have realistic deposition broadening
  cd_integrated = ec_data.cd_integrated.data;

  for i_lau = active_launchers
    % convert to j// with conversion factor
    cd_dens(:,i_lau,:) = squeeze(cd_dens(:,i_lau,:)).*jtoray_to_jpar0_mapped;
    for ii = 1:nt_ec_data
      % integrate j//
      cd_integrated(:,i_lau,ii) = cumtrapz(vol_mapped(:,ii),cd_dens(:,i_lau,ii))/(2*pi*R0);
    end
  end

%% interpolating 1d_profiles from 'ec_tgrid' grid (toray tgrid) to 'interp_tgrid' (powers tgrid when powers.data>0)
  ij = iround_os(interp_tgrid,ec_tgrid);
  sparse_p_ec_injected = interp_p_ec_injected(ij,:); % injected ec power vals corresponding to ec_tgrid
  n_rho = size(p_dens, 1);
  %rho grids to be interpolated on power time grid
  rho_pol_norm = ec_data.cd_dens.grids.rho_pol_norm;
  rho_tor_norm = ec_data.cd_dens.grids.rho_tor_norm;

  % calculate normalised profiles on ec_tgrid grid
  norm_p_dens        = zeros(n_rho,nb_launchers+1,nt_ec_data);
  norm_p_integrated  = zeros(n_rho,nb_launchers+1,nt_ec_data);
  norm_cd_dens       = zeros(n_rho,nb_launchers+1,nt_ec_data);
  norm_cd_integrated = zeros(n_rho,nb_launchers+1,nt_ec_data);
  for it = 1:nt_ec_data
    norm_p_dens(:,:,it)        = p_dens(:,:,it)./sparse_p_ec_injected(it,:);
    norm_p_integrated(:,:,it)  = p_integrated(:,:,it)./sparse_p_ec_injected(it,:);
    norm_cd_dens(:,:,it)       = cd_dens(:,:,it)./sparse_p_ec_injected(it,:);
    norm_cd_integrated(:,:,it) = cd_integrated(:,:,it)./sparse_p_ec_injected(it,:);
  end

  % interpolate normalised p_dens profiles on ec_powers_tgrid
  interp_norm_p_dens        = zeros(n_rho,nb_launchers+1,nt_interp);
  interp_norm_p_integrated  = zeros(n_rho,nb_launchers+1,nt_interp);
  interp_norm_cd_dens       = zeros(n_rho,nb_launchers+1,nt_interp);
  interp_norm_cd_integrated = zeros(n_rho,nb_launchers+1,nt_interp);
  % interpolate the rho_pol & rho_tor on ec_powers_tgrid
  interp_rho_pol             = zeros(n_rho,nt_interp);
  interp_rho_tor             = zeros(n_rho,nt_interp);
  interp_rho_flag = 1;
  for i_lau = active_launchers
    for irho = 1:n_rho
      % get power and current density at each rho
      trace_p_dens        = squeeze(norm_p_dens(irho,i_lau,:));
      trace_p_integrated  = squeeze(norm_p_integrated(irho,i_lau,:));
      trace_cd_dens       = squeeze(norm_cd_dens(irho,i_lau,:));
      trace_cd_integrated = squeeze(norm_cd_integrated(irho,i_lau,:));
      % interpolate on gdat powers tgrid
      interp_norm_p_dens(irho,i_lau,:)        = interpos(ec_tgrid,trace_p_dens,interp_tgrid);
      interp_norm_p_integrated(irho,i_lau,:)  = interpos(ec_tgrid,trace_p_integrated,interp_tgrid);
      interp_norm_cd_dens(irho,i_lau,:)       = interpos(ec_tgrid,trace_cd_dens,interp_tgrid);
      interp_norm_cd_integrated(irho,i_lau,:) = interpos(ec_tgrid,trace_cd_integrated,interp_tgrid);

      % only interpolate rho grids once
      if interp_rho_flag
        interp_rho_pol(irho,:) = interpos(ec_tgrid,rho_pol_norm(irho,:),interp_tgrid');
        interp_rho_tor(irho,:) = interpos(ec_tgrid,rho_tor_norm(irho,:),interp_tgrid');
      end
    end
    interp_rho_flag = 0;
  end

  % normalised & interpolated profiles * p_ec_injected on interp_tgrid
  interp_p_dens        = zeros(n_rho,nb_launchers+1,nt_interp);
  interp_p_integrated  = zeros(n_rho,nb_launchers+1,nt_interp);
  interp_cd_dens       = zeros(n_rho,nb_launchers+1,nt_interp);
  interp_cd_integrated = zeros(n_rho,nb_launchers+1,nt_interp);
  for it = 1:nt_interp
    interp_p_dens(:,:,it)        = interp_norm_p_dens(:,:,it).*interp_p_ec_injected(it,:);
    interp_p_integrated(:,:,it)  = interp_norm_p_integrated(:,:,it).*interp_p_ec_injected(it,:);
    interp_cd_dens(:,:,it)       = interp_norm_cd_dens(:,:,it).*interp_p_ec_injected(it,:);
    interp_cd_integrated(:,:,it) = interp_norm_cd_integrated(:,:,it).*interp_p_ec_injected(it,:);
  end

  % fill profiles_1d with interpolated profiles
  for i_lau = active_launchers
    % initialize var for current_parallel and coupled power which needs to be obatined from
    % integrated j// profiles and pdens last entry
    current_parallel_tmp = nan(1,nt_interp);
    power_coupled_tmp    = nan(1,nt_interp);

    for ii = 1:nt_interp
      % time
      ids_core_sources.source{last_index+i_lau}.profiles_1d{ii}.time = interp_tgrid(ii);
      % grids
      ids_core_sources.source{last_index+i_lau}.profiles_1d{ii}.grid.rho_tor_norm = interp_rho_tor(:,ii);
      ids_core_sources.source{last_index+i_lau}.profiles_1d{ii}.grid.rho_pol_norm = interp_rho_pol(:,ii);
      % power density
      ids_core_sources.source{last_index+i_lau}.profiles_1d{ii}.electrons.energy = ...
        interp_p_dens(:,i_lau,ii);
      % integrated power density
      ids_core_sources.source{last_index+i_lau}.profiles_1d{ii}.electrons.power_inside = ...
        interp_p_integrated(:,i_lau,ii);
      power_coupled_tmp(ii) = interp_p_integrated(end,i_lau,ii);
      % current density
      ids_core_sources.source{last_index+i_lau}.profiles_1d{ii}.j_parallel = ...
        interp_cd_dens(:,i_lau,ii);
      % integrated current density
      ids_core_sources.source{last_index+i_lau}.profiles_1d{ii}.current_parallel_inside = ...
        interp_cd_integrated(:,i_lau,ii);
      current_parallel_tmp(ii) = interp_cd_integrated(end,i_lau,ii);
    end
    ids_core_sources.source{last_index+i_lau}.globa_quantities.time = interp_tgrid;
    ids_core_sources.source{last_index+i_lau}.global_quantities.power = power_coupled_tmp;
    ids_core_sources.source{last_index+i_lau}.global_quantities.current_parallel = current_parallel_tmp;
  end

  %add empty sources for rest of unsused launchers
  if numel(ids_core_sources.source)-last_index ~= nb_launchers
    ids_core_sources.source{last_index+nb_launchers} = [];
  end

  last_index = last_index+nb_launchers;
end

%% nbi
% id_nbi.description = 'Source from Neutral Beam Injection';
% id_nbi.index = 2; id_nbi.name = 'nbi';
% ids_core_sources.source{} = source_template;
% ids_core_sources.source{}.identifier = id_nbi;
% ids_core_sources.source{}.profiles_1d(1:n_t) = {profiles_template};
% ids_core_sources.source{}.global_quantities(1:n_t) = {globals_template};

%% total
% id_total.description = 'Total source; combines all sources';
% id_total.index = 1; id_total.name = 'total';
% ids_core_sources.source{} = source_template;
% ids_core_sources.source{}.identifier = id_total;
% ids_core_sources.source{}.profiles_1d(1:n_t) = {profiles_template};
% ids_core_sources.source{}.global_quantities(1:n_t) = {globals_template};

%% add descriptions for profiles_1d

desc.source = 'available by now {source_index}: ohmic {1}, bootstrap {2}, ec {3+} for individual launchers (if present in shot)';
desc.profiles_1d.electrons.energy = 'absorbed power density';
desc.profiles_1d.electrons.power_inside = 'integrated absorbed power density';
desc.profiles_1d.j_parallel = 'parallel current density';
desc.profiles_1d.current_parallel_inside = 'integrated parallel current density';
desc.globals.power = 'total power coupled to the plasma';
desc.globals.current_parallel = 'total parallel current driven';
ids_core_sources_description = desc;

%%
if nargin <= 2
  ids_core_sources.code.name = ['tcv_get_ids_core_sources, within gdat, with shot= ' num2str(params_core_sources.shot) ];
else
  ids_core_sources.code.name = ['tcv_get_ids_core_sources, within gdat, with shot= ' num2str(params_core_sources.shot) '; varargin: ' varargin{:}];
end
ids_core_sources_description.code.name = ids_core_sources.code.name;

ids_core_sources.code.output_flag = zeros(size(ids_core_sources.time));

% cocos automatic transform
if ~isempty(which('ids_generic_cocos_nodes_transformation_symbolic'))
  [ids_core_sources,cocoscoeff]=ids_generic_cocos_nodes_transformation_symbolic(ids_core_sources,'core_sources',gdat_params.cocos_in, ...
          gdat_params.cocos_out,gdat_params.ipsign_out,gdat_params.b0sign_out,gdat_params.ipsign_in,gdat_params.b0sign_in, ...
          gdat_params.error_bar,gdat_params.nverbose);

end
